/***************************************************************************
 *   Copyright (C) 2003 by Dan Muller                                      *
 *   dan at verliba dot cz                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "cconndc.h"
#include "ckicklist.h"

namespace nDirectConnect {

namespace nTables {

cKickList::cKickList(cMySQL &mysql) : cConfMySQL(mysql)
{
	mTableName = "kicklist";
	SetClassName("cKickList");
	Add("nick", mModel.mNick);
	AddPrimaryKey("nick");
	Add("time", mModel.mTime);
	AddPrimaryKey("time");
	Add("ip", mModel.mIP);
	Add("host", mModel.mHost);
	Add("share_size", mModel.mShare);
	Add("email", mModel.mEmail);
	Add("reason", mModel.mReason);
	Add("op", mModel.mOp);
	Add("is_drop",mModel.mIsDrop);
	SetBaseTo(&mModel);
}

cKickList::~cKickList(){}
};
};

/*!
    \fn nDirectConnect::nTables::cKickList::Cleanup()
 */
void nDirectConnect::nTables::cKickList::Cleanup()
{
	mQuery.OStream() << "DELETE FROM " << mTableName << " WHERE time < " << cTime().Sec() - 24*3600*30;
	mQuery.Query();
	mQuery.Clear();
}


/*!
    \fn nDirectConnect::nTables::cKickList::CreateTable()
 */
void nDirectConnect::nTables::cKickList::CreateTable()
{
	mQuery.OStream() << "CREATE TABLE IF NOT EXISTS " << mTableName << " ( " <<
		"nick varchar(30) NOT NULL , "
		"time INTEGER NOT NULL, "
		"ip varchar(15), " // link to the ipbanlist table
		"host text, "
		"share_size varchar(15), "
		"email varchar(128), "
		"reason text, "
		"op varchar(30) NOT NULL, "
		"is_drop bool , "
		"PRIMARY KEY( nick, time ),"
		"INDEX op_index (op), "
		"INDEX ip_index (ip), "
		"INDEX drop_index (is_drop) "
		")";
	mQuery.Query();
	mQuery.Clear();
}



bool nDirectConnect::nTables::cKickList::AddKick(cConnDC *conn ,cConnDC *op, const string *reason, cKick &OldKick)
{
	if(!conn || !conn->mpUser) return false;
	if(!FindKick(OldKick, conn->mpUser->mNick,
		(op && op->mpUser)?op->mpUser->mNick:mEmpty,
		300,
		reason == NULL, reason != NULL ))
	{
		OldKick.mIP = conn->AddrIP();
		OldKick.mNick = conn->mpUser->mNick;
		if(op && op->mpUser) OldKick.mOp = op->mpUser->mNick;
		else OldKick.mOp = "VerliHub";
		OldKick.mTime = cTime().Sec();
		OldKick.mHost = conn->AddrHost();
		OldKick.mShare = conn->mpUser->mShare;
		OldKick.mEmail = conn->mpUser->mEmail;
		OldKick.mIsDrop = (reason == NULL);
	}
	if(reason) OldKick.mReason += *reason; else OldKick.mIsDrop = true;

	SetBaseTo(&OldKick);
	DeletePK();
	SavePK(false);
	return true;
}


/*!
    \fn nDirectConnect::nTables::cKickList::FindKick(cConnDC*, cConnDC*, unsigned)
 */
bool nDirectConnect::nTables::cKickList::FindKick(cKick &Kick, const string &Nick, const string &OpNick, unsigned age, bool WithReason, bool WithDrop, bool IsNick)
{
	ostringstream os;

	SelectFields(os);
	os <<" WHERE time > " << cTime().Sec()-age << " AND ";
	cConfigItemBase * item = NULL;
	string var;
	if(IsNick)
	{
		Kick.mNick = Nick;
		var = "nick";
	}
	else
	{
		Kick.mIP = Nick;
		var = "ip";
	}
	item = operator[](var);
	SetBaseTo(&Kick);

	ufEqual(os, string(" AND "), true, true, false)(item);

	os <<" AND reason IS " << (WithReason?"NOT ":"") << "NULL ";
	os <<" AND is_drop = " << WithDrop;
	if(OpNick.size())
		os << " AND op = '" << OpNick << "'";
	os <<" ORDER BY time DESC LIMIT 1";

	bool found = false;
	if( -1 != StartQuery(os.str()))
	{
		found = ( 0 <= Load());
		EndQuery();
	}
	return found;
}

